=begin
(c) TIG & SUPodium / Cadalog 2013-2014

SUPodium_Section_Tool_Loader.rb loads the main .rb[s] file from the Plugins/Cadalog 
subfolder.

Usage: With just one SectionPlane selected - from the context-menu use: 
  "SUPodium Section Tool"
Or in code use SUPodium_Section_Tool::Cut.new()

The Model will be Copied and then 'Cut' at that SectionPlane.
The original Model will remain unchanged by this 'Cut'.

Note that it is best to close, or at least rollup, the Outliner dialog before 
processing: at best it will slow processing considerably, at worst it could 
cause a BugSplat!

If you have an Internet connection then the dialogs differ slightly, because you 
can access online Help via the 'Help...' button; otherwise a dialog appears with 
some Help text.
Clicking a Help button leaves its dialog open, except for the final 'Done' 
dialog which will close as if 'Close' had been clicked.

On both PC and MAC:
You are reminded that the SKP will be Copied and 'Cut':
OK/Help/Cancel
The "Model.skp" is saved as "Model[x20130102_1234].skp".
Overly long file-names are suitably truncated - this can become necessary when 
recutting previously cut SKPs, as the name is repeatedly 'suffixed'.
The suffix's 'x' showing it is a 'Cut' version, with yyy/mm/dd_hrs/mins added, 
so that each Cut Model is saved uniquely named.

On a PC only, if the Model has been modified since its last 
Save you will be prompted to Save it by a SketchUp dialog.
Yes/No/Cancel
This is because the copied PC Model will open in the same 
window, thereby auto-closing the original potentially without 
saving it, but on a MAC a new window will open so the original 
always remains unaffected, and you should not be prompted.
Pressing 'Yes' will Save the modifications to the original, 
while 'No' does not, but if you 'Cancel' then the 'Cutting' 
process is aborted and the now redundant Copy is deleted.

Next...
The Copy is opened and processed...

Any Images are exploded and 'unused' things are also deleted - like Hidden 
objects [except Edges that are needed to define a still visible Face], objects 
on OFF Layers [except Edges that are needed to define a still visible Face], 
Construction Geometry, Text and Dimensions. 

The Model is then 'Cut' at that SectionPlane and everything in the 
active_context that is behind that plane is deleted: this allows you to 'cut'
 within Groups or Component-Instances that straddle the SectionPlane.
The 'cut-faces' are put into a new group named 'SUPodiumSectionCutFaces'.
These 'cut-faces' are given a material named 'SUPodiumSectionCutMaterial'; 
this is colored after the current Style's SectionPlane's default edge-color 
setting, unless a material using that name already exists.

If the Model appears to become 'unresponsive', the staus-bar progress reports 
seem to have frozen, or its window 'white-out' it does NOT mean that the 
processing has stalled or even failed.  When Sketchup is making complex 
calculations, changes to its geometry etc, it can close-down some of its 
display-functions to concentrate its efforts to be focused on the processing; 
if the Model is large, with a lot of 'Cut' geometry, or nested groups or 
component-instances [for example a Section in Plan cutting dozens of window-
components' frames] then the number of process -steps can easily run into many 
millions - so give it a chance to complete its processing before aborting and 
reporting an error - chill out have a coffee and wait for the 'beep'...

The processing ends with the new "Model[x20130102_1234].skp" still open and its 
changes saved.  There is a closing dialog: it has a 'Help...' button.
Close/Help

You can now edit the 'SUPodiumSectionCutFaces' group, or make a further 'Cut' 
using another Selected SectionPlane.

Note that ALL existing SectionPlanes are auto-assigned to an OFF Layer named 
'-OFFSECTS', so if recutting switch that layer on to see/select them.

It is recommended that if you are planning on making several 'Cuts' to the same 
Model, then you should 'Cut' SectionPlanes that are 'nested' first, so that 
any subsequent 'Cuts' will trim the earlier 'SUPodiumSectionCutFaces' groups 
properly.

Language Support:
There is a default .strings file in the Cadalog_Section subfolder, called
'SUPodium_Section_Tool-EN-US.strings'
This contains the strings to be used in dialogs etc.
For other locales an equivalent file can be added, for example in France:
'SUPodium_Section_Tool-FR.strings'
Note how the final '-EN-US' suffix is replaced with the equivalent '-FR'.
Each string is defined thus:
TOOLNAME==SUPodium Section Tool==
Only ever change the new .strings file's text to the right of the '=='.
The text in capitals is needed by the tool, it must not be changed.
For example for '-FR' you might make it:
TOOLNAME==SUPodium Outil de Section==
There are opening and closing '==' markers around the 'string'.  
There cannot be a '==' within a translated string!
The tool parses the .strings using these markers, to assemble the text for each 
situation, so the translated version should follow the same format and meaning.  
Some line lengths might need adjusting to keep dialogs looking acceptable should 
the translated text be significantly different in length.
Multiple lines are included, with the closing '==' after the last line.
You can use Sketchup.locale.upcase in the Ruby Console to get the current 
computer's locale.
If a computer has a locale for which there is no matching .strings file then 
the default '-EN-US' version is used.  If that is missing there's an error 
message and the tool cannot continue until there is a suitable .strings file 
in the Cadalog_Section folder.

If you have a SKP that splats when cutting you can set below...***
DEBUG=true to write a debug report-file, always kept after a 'splat'.
SAVE_REPORT=true to keep debug report-file even if there were no problems.
The default for both is =false.

Version:
01 20120306 First issue after beta-testing.
02 20140122 Future-proofed for 2014...
03 20140405 Saving copies is now v2014 compliant.
=end
require('sketchup.rb')
module SUPodium_Section_Tool
	###
	DEBUG=false #*** =true to write a debug report-file, kept after a 'splat'.
	SAVE_REPORT=false #*** =true to keep debug report-file even if no problems.
	###
	CADALOG_PATH = File.dirname(__FILE__)
	TOOL_NAME	 = "SUPodium Section Tool"
	HELP_URL	 = "http://www.suplugins.com/help/sections.php" ### TBC
	###
	Sketchup::require('Cadalog_Section/SUPodium_Section_Tool')
	###
	return nil unless self.set_strings() ### set up translated ,strings
	###
	### set up menu
	unless file_loaded?(__FILE__)
		UI.add_context_menu_handler{|menu|
			ss=Sketchup.active_model.selection
			if ss.length==1 && ss[0].is_a?(Sketchup::SectionPlane)
				menu.add_item(TOOLNAME){SUPodium_Section_Tool::Cut.new()}
			end
		}
	end
	file_loaded(__FILE__)
end
